@@ -1,13 +1,12 @@
 import esphome.codegen as cg
 import esphome.config_validation as cv
 from esphome.components import ble_client, esp32_ble_tracker, output
-from esphome.const import CONF_ID, CONF_SERVICE_UUID
+from esphome.const import CONF_CHARACTERISTIC_UUID, CONF_ID, CONF_SERVICE_UUID
 
 from .. import ble_client_ns
 
 DEPENDENCIES = ["ble_client"]
 
-CONF_CHARACTERISTIC_UUID = "characteristic_uuid"
 CONF_REQUIRE_RESPONSE = "require_response"
 
 BLEBinaryOutput = ble_client_ns.class_(
