@@ -1,7 +1,6 @@
 import logging
 from importlib import resources
 from typing import Optional
-from datetime import timezone
 
 import tzlocal
 
@@ -66,14 +65,11 @@ def _extract_tz_string(tzfile: bytes) -> str:
 
 
 def detect_tz() -> str:
-    localzone = tzlocal.get_localzone()
-    if localzone is timezone.utc:
-        return "UTC0"
-    if not hasattr(localzone, "key"):
+    iana_key = tzlocal.get_localzone_name()
+    if iana_key is None:
         raise cv.Invalid(
             "Could not automatically determine timezone, please set timezone manually."
         )
-    iana_key = localzone.key
     _LOGGER.info("Detected timezone '%s'", iana_key)
     tzfile = _load_tzdata(iana_key)
     if tzfile is None:
@@ -130,10 +126,10 @@ def _parse_cron_part(part, min_value, max_value, special_mapping):
             )
         begin, end = data
         begin_n = _parse_cron_int(
-            begin, special_mapping, "Number for time range must be integer, " "got {}"
+            begin, special_mapping, "Number for time range must be integer, got {}"
         )
         end_n = _parse_cron_int(
-            end, special_mapping, "Number for time range must be integer, " "got {}"
+            end, special_mapping, "Number for time range must be integer, got {}"
         )
         if end_n < begin_n:
             return set(range(end_n, max_value + 1)) | set(range(min_value, begin_n + 1))
@@ -143,7 +139,7 @@ def _parse_cron_part(part, min_value, max_value, special_mapping):
         _parse_cron_int(
             part,
             special_mapping,
-            "Number for time expression must be an " "integer, got {}",
+            "Number for time expression must be an integer, got {}",
         )
     }
 
