@@ -10,6 +10,11 @@ static const char *const TAG = "tuya.light";
 void TuyaLight::setup() {
   if (this->color_temperature_id_.has_value()) {
     this->parent_->register_listener(*this->color_temperature_id_, [this](const TuyaDatapoint &datapoint) {
+      if (this->state_->current_values != this->state_->remote_values) {
+        ESP_LOGD(TAG, "Light is transitioning, datapoint change ignored");
+        return;
+      }
+
       auto datapoint_value = datapoint.value_uint;
       if (this->color_temperature_invert_) {
         datapoint_value = this->color_temperature_max_value_ - datapoint_value;
@@ -23,6 +28,11 @@ void TuyaLight::setup() {
   }
   if (this->dimmer_id_.has_value()) {
     this->parent_->register_listener(*this->dimmer_id_, [this](const TuyaDatapoint &datapoint) {
+      if (this->state_->current_values != this->state_->remote_values) {
+        ESP_LOGD(TAG, "Light is transitioning, datapoint change ignored");
+        return;
+      }
+
       auto call = this->state_->make_call();
       call.set_brightness(float(datapoint.value_uint) / this->max_value_);
       call.perform();
@@ -30,6 +40,11 @@ void TuyaLight::setup() {
   }
   if (switch_id_.has_value()) {
     this->parent_->register_listener(*this->switch_id_, [this](const TuyaDatapoint &datapoint) {
+      if (this->state_->current_values != this->state_->remote_values) {
+        ESP_LOGD(TAG, "Light is transitioning, datapoint change ignored");
+        return;
+      }
+
       auto call = this->state_->make_call();
       call.set_state(datapoint.value_bool);
       call.perform();
@@ -41,6 +56,11 @@ void TuyaLight::setup() {
       auto green = parse_hex<uint8_t>(datapoint.value_string.substr(2, 2));
       auto blue = parse_hex<uint8_t>(datapoint.value_string.substr(4, 2));
       if (red.has_value() && green.has_value() && blue.has_value()) {
+        if (this->state_->current_values != this->state_->remote_values) {
+          ESP_LOGD(TAG, "Light is transitioning, datapoint change ignored");
+          return;
+        }
+
         auto call = this->state_->make_call();
         call.set_rgb(float(*red) / 255, float(*green) / 255, float(*blue) / 255);
         call.perform();
@@ -52,6 +72,11 @@ void TuyaLight::setup() {
       auto saturation = parse_hex<uint16_t>(datapoint.value_string.substr(4, 4));
       auto value = parse_hex<uint16_t>(datapoint.value_string.substr(8, 4));
       if (hue.has_value() && saturation.has_value() && value.has_value()) {
+        if (this->state_->current_values != this->state_->remote_values) {
+          ESP_LOGD(TAG, "Light is transitioning, datapoint change ignored");
+          return;
+        }
+
         float red, green, blue;
         hsv_to_rgb(*hue, float(*saturation) / 1000, float(*value) / 1000, red, green, blue);
         auto call = this->state_->make_call();
@@ -71,31 +96,34 @@ void TuyaLight::dump_config() {
     ESP_LOGCONFIG(TAG, "   Dimmer has datapoint ID %u", *this->dimmer_id_);
   if (this->switch_id_.has_value())
     ESP_LOGCONFIG(TAG, "   Switch has datapoint ID %u", *this->switch_id_);
-  if (this->rgb_id_.has_value())
+  if (this->rgb_id_.has_value()) {
     ESP_LOGCONFIG(TAG, "   RGB has datapoint ID %u", *this->rgb_id_);
-  else if (this->hsv_id_.has_value())
+  } else if (this->hsv_id_.has_value()) {
     ESP_LOGCONFIG(TAG, "   HSV has datapoint ID %u", *this->hsv_id_);
+  }
 }
 
 light::LightTraits TuyaLight::get_traits() {
   auto traits = light::LightTraits();
   if (this->color_temperature_id_.has_value() && this->dimmer_id_.has_value()) {
     if (this->rgb_id_.has_value() || this->hsv_id_.has_value()) {
-      if (this->color_interlock_)
+      if (this->color_interlock_) {
         traits.set_supported_color_modes({light::ColorMode::RGB, light::ColorMode::COLOR_TEMPERATURE});
-      else
+      } else {
         traits.set_supported_color_modes(
             {light::ColorMode::RGB_COLOR_TEMPERATURE, light::ColorMode::COLOR_TEMPERATURE});
+      }
     } else
       traits.set_supported_color_modes({light::ColorMode::COLOR_TEMPERATURE});
     traits.set_min_mireds(this->cold_white_temperature_);
     traits.set_max_mireds(this->warm_white_temperature_);
   } else if (this->rgb_id_.has_value() || this->hsv_id_.has_value()) {
     if (this->dimmer_id_.has_value()) {
-      if (this->color_interlock_)
+      if (this->color_interlock_) {
         traits.set_supported_color_modes({light::ColorMode::RGB, light::ColorMode::WHITE});
-      else
+      } else {
         traits.set_supported_color_modes({light::ColorMode::RGB_WHITE});
+      }
     } else
       traits.set_supported_color_modes({light::ColorMode::RGB});
   } else if (this->dimmer_id_.has_value()) {
